// Qt includes
#include <QApplication>
#include <QDateTime>
#include <QDebug>
#include <QWidget>
#include <QMouseEvent>

#ifdef _MSC_VER
#include <windows.h>
#endif

class EventTesterWidget : public QWidget
{
public:
    EventTesterWidget(QWidget *parent = nullptr) : QWidget(parent)
    {
        // This results in lots of output as soon as the pointer is over the widget
        //setMouseTracking(true);
    }

protected:
    void mousePressEvent(QMouseEvent *event) override
    {
        qDebug() << "Mouse press event";
        inspectMouseEvent(event);
    }

    void mouseMoveEvent(QMouseEvent *event) override
    {
        qDebug() << "Mouse move event";
        inspectMouseEvent(event);
    }

    void mouseReleaseEvent(QMouseEvent *event) override
    {
        qDebug() << "Mouse release event";
        inspectMouseEvent(event);
    }

    void mouseDoubleClickEvent(QMouseEvent *event) override
    {
        qDebug() << "Mouse double click event";
        inspectMouseEvent(event);
    }

private:
    void inspectMouseEvent(QMouseEvent *event)
    {
        qDebug() << "Event time:" << QDateTime::currentDateTime();

        const auto device = event->device();
        qDebug() << "Name:" << device->name();
        qDebug() << "Seat name:" << device->seatName();
        qDebug() << "Type:" << device->type();
        qDebug() << "";
    }
};


int startGUI(int argc, char *argv[])
{
    QApplication app(argc, argv);

    // Widgets
    EventTesterWidget etw;
    etw.setWindowTitle("Event Testing Widget");
    etw.show();

    return app.exec();
}

// In case we compile with MSVC WinMain is expected as the entry point
// #ifdef _MSC_VER
// int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
//{
//	return startGUI(0, 0);
//}
// #else // GCC and MinGW use the "normal" main function.
int main(int argc, char *argv[])
{
    startGUI(argc, argv);
}
// #endif
